/*
 * File : dlFrameReject.C
 * 
 * Description : This file contains the method definitions for the
 * dlFrameReject class.
 */

#include "types.h"
#include "frame.h"
#include "dataPacket.h"
#include "phEvents.h"
#include "dlEvents.h"
#include "dllayer.h"
#include "dlFrameReject.h"
#include "dlInfoXfer.h"
#include "dlDisconnected.h"
#include "dlDiscPending.h"
#include "dlOutSetupPending.h"
#include "timerT1.h"

/*
 * Function : dlFrameReject()
 *
 * Description : This is the constructor. It calls the base class
 * constructor.
 */

dlFrameReject::dlFrameReject(dllayer *dl) : dlStateMachine(dl) 
{
}

/*
 * Function : PrintState()
 *
 * Description : Prints the state.
 */

void 
dlFrameReject::PrintState() 
{
	TRACE(dlSmModule,1,("State dlFrameReject\n"));
}

/*
 * Function : HandleSabm()
 *
 * Description : Processes a SABM that has been received.
 *	Transmits UA response, issues a DL_ResetInd to the
 *	network layer, and changes state back to dlInfoXfer.
 */

dlStateMachine *
dlFrameReject::HandleSabm(int &errorCode) 
{
	TRACE(dlSmModule,1,("In HandleSabm\n"));
	_dl->XmitFrame(errorCode,FALSE,UA,_dl->_lastFrameRecd->GetPf());
	_dl->_nPlus1->Command(DL_ResetInd,errorCode,internalError);
	CHANGE_TO_INFOXFER(TRUE);
}

/*
 * Function : HandleDisc()
 *
 * Description : Processes a DISC that has been received.
 *	Transmits a UA, and changes state to dlDisconnected.
 *	Invokes a DL_DisconnectInd on the network layer.
 */

dlStateMachine *
dlFrameReject::HandleDisc(int &errorCode) 
{
	TRACE(dlSmModule,1,("In HandleDisc\n"));
	_dl->XmitFrame(errorCode,FALSE,UA,_dl->_lastFrameRecd->GetPf());
	_dl->_nPlus1->Command(DL_DisconnectInd,errorCode,internalError);
	CHANGE_STATE(dlDisconnected);
}

/*
 * Function : HandleDm()
 *
 * Description : Processes a DM that has been received.
 * Ignore DM (f=1). 
 * For DM, F= 0 which is a mode-setting command, start
 * link reset process by transmitting a SABM and changing
 * state to dlOutSetupPending.
 */


dlStateMachine *
dlFrameReject::HandleDm(int &errorCode)
{
	// Ignore DM F=1, we should not even be getting this.
	if ( _dl->_lastFrameRecd->GetPf())
		return this;

	// If it is a mode setting command - start link reset

	_dl->XmitFrame(errorCode,TRUE,SABM,1);
	_dl->StartT1Timer(FALSE);					
	_dl->_setupType = internalReset; // Note setup type is internal
	CHANGE_STATE(dlOutSetupPending);
}

/*
 * Function : HandleFrmr()
 *
 * Description : Processes a FRMR that has been received. 
 *	Start a link reset by transmitting a SABM, and changing
 * 	state to dlOutSetupPending.
 */

dlStateMachine *
dlFrameReject::HandleFrmr(int &errorCode)
{
	TRACE(dlSmModule,1,("In HandleFrmr\n"));

	_dl->XmitFrame(errorCode,TRUE,SABM,1);
	_dl->StartT1Timer(FALSE);					
	_dl->_setupType = internalReset; // note setup type is internal
	CHANGE_STATE(dlOutSetupPending);
} 

/*
 * Function : HandleT1Timer()
 * 
 * Description : Processes a timer T1 expiry event. If FRMR has
 * been retransmitted less tha N2 times, retransmit it again.
 * Else, start link reset proceedings by transmitting a SABM,
 * and changing state to dlOutSetupPending.
 */

dlStateMachine *
dlFrameReject::HandleT1Timer(int &errorCode)
{
	TRACE(dlSmModule,1,("dlFrameReject: In HandleT1\n"));
	if ( _dl->_t1RetryCount < N2 ) {
		// Re-transmit a FRMR
		_dl->XmitFrmr(0,errorCode);
		if ( errorCode == codeSuccessful ) {		
			// Re-start T1 timer.
			_dl->StartT1Timer(TRUE);					
			return this;
		}
		else {
			// Failed to transmit FRMR
			TRACE(dlSmModule,1,("Failed to transmit FRMR!\n"));
			_dl->_nPlus1->Command(DL_DisconnectInd,errorCode,noPeerResponse);
			CHANGE_STATE(dlDisconnected);
		}
	}
	else { 
		// Retransmissions exceed N2, disconnect link
		TRACE(dlSmModule,1,("Timer T1 timed out N2 times\n"));
		_dl->XmitFrame(errorCode,TRUE,SABM,1);
		_dl->StartT1Timer(FALSE);					
		_dl->_setupType = internalReset;
		CHANGE_STATE(dlOutSetupPending);
	}
}

/*
 * Function : DlResetReq()
 *
 * Description : Handles a DL_ResetReq primitive invoked by
 * the network layer. Transmit SABM, change state to
 * dlOutSetupPending.
 */


dlStateMachine *
dlFrameReject::DlResetReq(int &errorCode) 
{
	TRACE(dlSmModule,1,("Entering DlResetReq\n"));

	// now, go ahead and send a SABM

	_dl->XmitFrame(errorCode,TRUE,SABM,1);
	if ( errorCode == codeSuccessful ) {		
		_dl->StartT1Timer(FALSE);					
		_dl->_setupType = externalReset;
		CHANGE_STATE(dlOutSetupPending);
	}
	else 
		return this;
}


/*
 * Function : DlDisconnectReq()
 *
 * Description : Handles a DL_DisconnectReq primitive invoked
 *	by the network layer. Transmit a DISC, and change
 *	state to dlDiscPending.
 */

dlStateMachine *
dlFrameReject::DlDisconnectReq(int &errorCode) 
{
	TRACE(dlSmModule,1,("Entering DlDisconnectReq\n"));
	// send a DISC

	_dl->XmitFrame(errorCode,TRUE,DISC,1);

	if ( errorCode == codeSuccessful ) {		
		_dl->StartT1Timer(FALSE);					
		CHANGE_STATE(dlDiscPending);
	}
	else 
		return this;
}

/*
 * Function : HandleRr()
 *
 * Description : Processes a RR that was received.
 * Invoke HandleCmd()
 */

dlStateMachine *
dlFrameReject::HandleRr(int &errorCode)
{
	HandleCmd(errorCode);
	return this;
}
 
/*
 * Function : HandleRnr()
 *
 * Description : Processes a RNR that was received.
 * Invoke HandleCmd()
 */

dlStateMachine *
dlFrameReject::HandleRnr(int &errorCode)
{
	HandleCmd(errorCode);
	return this;
}
 
/*
 * Function : HandleRej()
 *
 * Description : Processes a REJ that was received.
 * Invoke HandleCmd()
 */

dlStateMachine *
dlFrameReject::HandleRej(int &errorCode)
{
	HandleCmd(errorCode);
	return this;
}

/*
 * Function : HandleIframe()
 *
 * Description : Processes a I-frame that was received.
 * Invoke HandleCmd()
 */

dlStateMachine *
dlFrameReject::HandleIframe(int &errorCode)
{
	HandleCmd(errorCode);
	return this;
}
 
/*
 * Function : HandleCmd()
 *
 * Description : If received frame is a command, and
 *	P bit = 1, then transmit a FRMR response. 
 */

void
dlFrameReject::HandleCmd(int &errorCode)
{
	if ( _dl->_lastFrameRecd->IsCmd() == TRUE && 
		_dl->_lastFrameRecd->GetPf()) {
		_dl->XmitFrmr(1,errorCode);
	}	
}

/*
 * Function : HandleUndefined()
 *
 * Description : Processes a undefined frame that was received.
 * Invoke HandleCmd()
 */

dlStateMachine *
dlFrameReject::HandleUndefined(int &errorCode)
{
	HandleCmd(errorCode);
	return this;
}

/*
 * Function : HandleIframe()
 *
 * Description : Processes a frame with incorrect length
 *	that was received. Invoke HandleCmd()
 */

dlStateMachine *
dlFrameReject::HandleIncorrectLength(int &errorCode)
{
	HandleCmd(errorCode);
	return this;
}

/*
 * Function : HandleIframe()
 *
 * Description : Processes a I-frame that was received and
 *	whose length exceeds the maximum limit.
 * Invoke HandleCmd()
 */

dlStateMachine *
dlFrameReject::HandleIfrLengthExcess(int &errorCode)
{
	HandleCmd(errorCode);
	return this;
}

