/*
 * File : dlStateMachine.C
 *
 * This file contains the method definitions for the class dlStateMachine.
 */

#include "types.h"
#include "dllayer.h"
#include "dlEvents.h"
#include "dlStateMachine.h"
#include "dlDisconnected.h"

/*
 * Define the trace module object.
 */

DEFINE_TRACE_MODULE(dlSmModule,"dlSM");

/*
 * Function : dlStateMachine()
 *
 * Description : This is the constructor. It initializes private
 *	variables.
 */

dlStateMachine::dlStateMachine(dllayer *dl) : _dl(dl)
{
}

/*
 * Function : PhActivateInd()
 *
 * Description : This function handles the primitive Ph_ActivateInd
 *	invoked by the physical layer. Merely returns successful. We
 *	really don't need to do anything.
 */

dlStateMachine *
dlStateMachine::PhActivateInd(int &errorCode)
{
	errorCode = codeSuccessful;
	return this;
}
		
/*
 * Function : PhDeactivateInd()
 *
 * Description : This function handles the primitive 
 * Ph_DeactivateInd invoked by the physical layer. 
 * It invokes the DL_DisconnectInd primitive on the 
 * network layer, and changes state to Disconnected.
 */

dlStateMachine *
dlStateMachine::PhDeactivateInd(int &errorCode)
{
	_dl->_nPlus1->Command(DL_DisconnectInd,errorCode,peerDisconnect);
	CHANGE_STATE(dlDisconnected);
}

/*
 * The following functions provide the default 
 * implementations for the event handling functions - 
 * merely printing out messages.
 */

dlStateMachine *
dlStateMachine::DlConnectReq(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Ignoring Connect Request\n"));
	errorCode = codeGeneralFailure; 
	return this;
}

dlStateMachine *
dlStateMachine::DlConnectRes(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Ignoring Connect Response\n"));
	errorCode = codeGeneralFailure; 
	return this;
}

dlStateMachine *
dlStateMachine::DlDisconnectReq(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Ignoring Discoonnect Request\n"));
	errorCode = codeGeneralFailure; 
	return this;
}

dlStateMachine *
dlStateMachine::DlDataReq(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Ignoring Data Request\n"));
	errorCode = codeGeneralFailure; 
	return this;
}

dlStateMachine *
dlStateMachine::DlResetReq(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Ignoring Reset Request\n"));
	errorCode = codeGeneralFailure; 
	return this;
}

dlStateMachine *
dlStateMachine::DlResetRes(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Ignoring Reset Response\n"));
	errorCode = codeGeneralFailure; 
	return this;
}


dlStateMachine *
dlStateMachine::HandleIframe(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Discarding IFRAME\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleSabm(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Discarding SABM\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleDisc(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Discarding DISC\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleDm(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Discarding DM\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleUa(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Discarding UA\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleRr(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Discarding RR\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleRnr(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Discarding RNR\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleRej(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Discarding REJ\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleFrmr(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Discarding FRMR\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleUndefined(int &errorCode) 
{ 
	TRACE(dlSmModule,1,("Ignoring Undefined frame\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleUnsolFBit(int &errorCode)
{ 
	TRACE(dlSmModule,1,("Ignoring Unsolicited F-Bit\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleIncorrectLength(int &errorCode)
{ 
	TRACE(dlSmModule,1,("Ignoring incorrect length frame\n"));
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleIfrLengthExcess(int &errorCode)
{ 
	TRACE(dlSmModule,1,("Discarding IFRAME with excess length\n"));
	errorCode = codeSuccessful; 
	return this;
}


// Internal events

dlStateMachine *
dlStateMachine::HandleT1Timer(int &errorCode)
{ 
	TRACE(dlSmModule,1,("Ignoring  T1 Timer\n"));
	_dl->_t1Timer = 0;
	errorCode = codeSuccessful; 
	return this;
}

dlStateMachine *
dlStateMachine::HandleRejTimer(int &errorCode)
{ 
	TRACE(dlSmModule,1,("Ignoring REJ timer\n"));
	_dl->_rejTimer = 0;
	errorCode = codeSuccessful; 
	return this;
}


/*
 * Function : PrintState()
 *
 * Description : Prints the state.
 */

void
dlStateMachine::PrintState()
{
	TRACE(dlSmModule,1,("In dlStateMachine\n"));
}

